@@ -727,11 +727,11 @@ autoar_extractor_signal_decide_destination (AutoarExtractor *self,
                                             GList *files,
                                             GFile **new_destination)
 {
-  g_signal_emit (self,
-                 autoar_extractor_signals[DECIDE_DESTINATION], 0,
-                 destination,
-                 files,
-                 new_destination);
+  autoar_common_g_signal_emit (self, self->in_thread,
+                               autoar_extractor_signals[DECIDE_DESTINATION], 0,
+                               destination,
+                               files,
+                               new_destination);
 }
 
 static inline void
@@ -845,13 +845,16 @@ autoar_extractor_get_common_prefix (GList *files,
 
 static GFile*
 autoar_extractor_do_sanitize_pathname (AutoarExtractor *self,
-                                       const char      *pathname)
+                                       const char      *pathname_bytes)
 {
   GFile *extracted_filename;
   gboolean valid_filename;
   g_autofree char *sanitized_pathname;
+  g_autofree char *utf8_pathname;
 
-  extracted_filename = g_file_get_child (self->destination_dir, pathname);
+  utf8_pathname = autoar_common_get_utf8_pathname (pathname_bytes);
+  extracted_filename = g_file_get_child (self->destination_dir,
+                                         utf8_pathname ?  utf8_pathname : pathname_bytes);
 
   valid_filename =
     g_file_equal (extracted_filename, self->destination_dir) ||
@@ -1582,6 +1585,7 @@ autoar_extractor_step_scan_toplevel (AutoarExtractor *self)
 
   while ((r = archive_read_next_header (a, &entry)) == ARCHIVE_OK) {
     const char *pathname;
+    g_autofree char *utf8_pathname = NULL;
 
     if (g_cancellable_is_cancelled (self->cancellable)) {
       archive_read_free (a);
@@ -1589,27 +1593,42 @@ autoar_extractor_step_scan_toplevel (AutoarExtractor *self)
     }
 
     if (archive_entry_is_encrypted (entry)) {
-      if (self->error == NULL) {
-        self->error = g_error_new (G_IO_ERROR,
-                                   G_IO_ERROR_NOT_SUPPORTED,
-                                   "Encrypted archives are not supported.");
-      }
-      archive_read_free (a);
-      return;
+      break;
     }
 
-    pathname = archive_entry_pathname (entry);
-    g_debug ("autoar_extractor_step_scan_toplevel: %d: pathname = %s",
-             self->total_files, pathname);
+    if (self->use_raw_format) {
+      pathname = autoar_common_get_basename_remove_extension (g_file_get_path(self->source_file));
+      g_debug ("autoar_extractor_step_scan_toplevel: %d: raw pathname = %s",
+               self->total_files, pathname);
+    } else {
+      pathname = archive_entry_pathname (entry);
+      utf8_pathname = autoar_common_get_utf8_pathname (pathname);
 
+      g_debug ("autoar_extractor_step_scan_toplevel: %d: pathname = %s %s%s",
+               self->total_files, pathname,
+               utf8_pathname ? "utf8 pathname = " :"",
+               utf8_pathname ? utf8_pathname : "");
+    }
     self->files_list =
       g_list_prepend (self->files_list,
-                      g_file_get_child (self->output_file, pathname));
+                      g_file_get_child (self->output_file,
+                                        utf8_pathname ? utf8_pathname : pathname));
     self->total_files++;
     self->total_size += archive_entry_size (entry);
     archive_read_data_skip (a);
   }
 
+  if (entry && archive_entry_is_encrypted (entry)) {
+    g_debug ("autoar_extractor_step_scan_toplevel: encrypted entry");
+    if (self->error == NULL) {
+      self->error = g_error_new (G_IO_ERROR,
+                                 G_IO_ERROR_NOT_SUPPORTED,
+                                 "Encrypted archives are not supported.");
+    }
+    archive_read_free (a);
+    return;
+  }
+
   if (self->files_list == NULL) {
     if (self->error == NULL) {
       self->error = g_error_new (AUTOAR_EXTRACTOR_ERROR,
@@ -1681,7 +1700,8 @@ autoar_extractor_step_set_destination (AutoarExtractor *self)
     prefix_name = g_file_get_basename (self->prefix);
     prefix_name_no_ext = autoar_common_get_basename_remove_extension (prefix_name);
 
-    if (g_strcmp0 (prefix_name_no_ext, self->suggested_destname) == 0) {
+    if (g_strcmp0 (prefix_name, self->suggested_destname) == 0 ||
+        g_strcmp0 (prefix_name_no_ext, self->suggested_destname) == 0) {
       self->destination_dir = g_object_ref (self->output_file);
     } else {
       g_clear_object (&self->prefix);
